#ifndef CONFIGURATION_H
#define CONFIGURATION_H

// This configurtion file contains the basic settings.
// Advanced settings can be found in Configuration_adv.h 
// BASIC SETTINGS: select your board type, temperature sensor type, axis scaling, and endstop configuration

//User specified version info of this build to display in [Pronterface, etc] terminal window during startup.
//Implementation of an idea by Prof Braino to inform user that any changes made
//to this build by the user have been successfully uploaded into firmware.
#define STRING_VERSION_CONFIG_H __DATE__ " " __TIME__ // build date and time
#define STRING_CONFIG_H_AUTHOR "German RepRap" //Who made the changes.

// Languages
// 1  English
// 2  Polish
// 3  French	(awaiting translation!)
// 4  German
// 5  Spanish
// 6  Russian
// 7  Italian
// 8  Portuguese
// 9  Finnish

#define LANGUAGE_CHOICE 4

// SERIAL_PORT selects which serial port should be used for communication with the host.
// This allows the connection of wireless adapters (for instance) to non-default port pins.
// Serial port 0 is still used by the Arduino bootloader regardless of this setting.
#define SERIAL_PORT 0

// This determines the communication speed of the printer
#define BAUDRATE 115000

//// The following define selects which electronics board you have. Please choose the one that matches your setup
// PRotos RAMPS 1.2 = 3
// PRotos RAMPS 1.3/1.4 = 33 one Extruder (Power outputs: Extruder, Bed, Fan)
// PRotos X400 RAMPS 1.3/1.4 = 35 one Extruder (Power outputs: Extruder, Bed, Fan)
// Rambo Board = 301 (experimental)
#define MOTHERBOARD 33

//// The following is your used extruder type
// Hinged/Wade Exruder = 1
// DD-Extruder = 2
// DD-Extruder dual = 3
// DD2 Extruder = 4
#define EXTRUDER 2

//LCD and SD support
//#define ULTRA_LCD  // lcd support
#define BEEPER_OFF // turn off beeper on LCD 
//#define SDSUPPORT // Enable SD Card Support in Hardware Console

//// The following define selects which power supply you have. Please choose the one that matches your setup
// 1 = ATX

#define POWER_SUPPLY 1

//===========================================================================
//=============================Thermal Settings  ============================
//===========================================================================
//
//--NORMAL IS 4.7kohm PULLUP!-- 1kohm pullup can be used on hotend sensor, using correct resistor and table
//
//// Temperature sensor settings:

// 1 is 100k thermistor

#define TEMP_SENSOR_0 1
#define TEMP_SENSOR_1 0
#define TEMP_SENSOR_2 0
#define TEMP_SENSOR_BED 1

// Actual temperature must be close to target for this long before M109 returns success
#define TEMP_RESIDENCY_TIME 10	// (seconds)
#define TEMP_HYSTERESIS 3       // (degC) range of +/- temperatures considered "close" to the target one
#define TEMP_WINDOW     1       // (degC) Window around target to start the recidency timer x degC early.

// The minimal temperature defines the temperature below which the heater will not be enabled It is used
// to check that the wiring to the thermistor is not broken. 
// Otherwise this would lead to the heater being powered on all the time.
#define HEATER_0_MINTEMP 5
#define HEATER_1_MINTEMP 5
#define HEATER_2_MINTEMP 5
#define BED_MINTEMP 5

// When temperature exceeds max temp, your heater will be switched off.
// This feature exists to protect your hotend from overheating accidentally, but *NOT* from thermistor short/failure!
// You should use MINTEMP for thermistor short/failure protection.
#define HEATER_0_MAXTEMP 275
#define HEATER_1_MAXTEMP 275
#define HEATER_2_MAXTEMP 275
#define BED_MAXTEMP 180

// If your bed has low resistance e.g. .6 ohm and throws the fuse you can duty cycle it to reduce the
// average current. The value should be an integer and the heat bed will be turned on for 1 interval of
// HEATER_BED_DUTY_CYCLE_DIVIDER intervals.
//#define HEATER_BED_DUTY_CYCLE_DIVIDER 4

// PID settings:
// Comment the following line to disable PID and enable bang-bang.
#define PIDTEMP
#define PID_MAX 256 // limits current to nozzle; 256=full current
#ifdef PIDTEMP
  //#define PID_DEBUG // Sends debug data to the serial port. 
  //#define PID_OPENLOOP 1 // Puts PID in open loop. M104/M140 sets the output power from 0 to PID_MAX
  #define PID_FUNCTIONAL_RANGE 10 // If the temperature difference between the target temperature and the actual temperature
                                  // is more then PID_FUNCTIONAL_RANGE then the PID will be shut off and the heater will be set to min/max.
  #define PID_INTEGRAL_DRIVE_MAX 255  //limit for the integral term
  #define K1 0.95 //smoothing factor withing the PID
  #define PID_dT ((16.0 * 8.0)/(F_CPU / 64.0 / 256.0)) //sampling period of the temperature routine

  #define  DEFAULT_Kp 58.37
  #define  DEFAULT_Ki 9.00
  #define  DEFAULT_Kd 94.68

#endif // PIDTEMP

// Bed Temperature Control
// Select PID or bang-bang with PIDTEMPBED.  If bang-bang, BED_LIMIT_SWITCHING will enable hysteresis
//
// uncomment this to enable PID on the bed.   It uses the same ferquency PWM as the extruder. 
// If your PID_dT above is the default, and correct for your hardware/configuration, that means 7.689Hz,
// which is fine for driving a square wave into a resistive load and does not significantly impact you FET heating.
// This also works fine on a Fotek SSR-10DA Solid State Relay into a 250W heater. 
// If your configuration is significantly different than this and you don't understand the issues involved, you proabaly 
// shouldn't use bed PID until someone else verifies your hardware works.
// If this is enabled, find your own PID constants below.
//#define PIDTEMPBED
//
//#define BED_LIMIT_SWITCHING

// This sets the max power delived to the bed, and replaces the HEATER_BED_DUTY_CYCLE_DIVIDER option.
// all forms of bed control obey this (PID, bang-bang, bang-bang with hysteresis)
// setting this to anything other than 256 enables a form of PWM to the bed just like HEATER_BED_DUTY_CYCLE_DIVIDER did,
// so you shouldn't use it unless you are OK with PWM on your bed.  (see the comment on enabling PIDTEMPBED)
#define MAX_BED_POWER 256 // limits duty cycle to bed; 256=full current

//this prevents dangerous Extruder moves, i.e. if the temperature is under the limit
//can be software-disabled for whatever purposes by
#define PREVENT_DANGEROUS_EXTRUDE
//if PREVENT_DANGEROUS_EXTRUDE is on, you can still disable (uncomment) very long bits of extrusion separately.
#define PREVENT_LENGTHY_EXTRUDE

#define EXTRUDE_MINTEMP 30
#define EXTRUDE_MAXLENGTH (X_MAX_LENGTH+Y_MAX_LENGTH) //prevent extrusion of very large distances.

//===========================================================================
//=============================Mechanical Settings===========================
//===========================================================================

// Printer type
// PRotos = 1
// PRotos X400 = 2
#if MOTHERBOARD == 3
	#define PRINTER_TYPE 1
#endif
#if MOTHERBOARD == 33
	#define PRINTER_TYPE 1
#endif
#if MOTHERBOARD == 35
	#define PRINTER_TYPE 2
#endif
#if MOTHERBOARD == 301
	#define PRINTER_TYPE 1
#endif

 
// Extruder type
#if EXTRUDER == 1
        #define EXTRUDER_STEPS 172.222
        #define EXTRUDER_DEFAULT_MAX_FEEDRATE 45
        #define EXTRUDER_DEFAULT_MAX_ACCELERATION 10000
        #define EXTRUDER_DEFAULT_EJERK 5.0 // (mm/sec)
        #define INVERT_E0_DIR false
        #define INVERT_E1_DIR false
        #define INVERT_E2_DIR false
        
        // This defines the number of extruders, set to 2 for DD-Extruder dual
        #define EXTRUDERS 1
#endif
#if EXTRUDER >= 2
        #define EXTRUDER_STEPS 99
        #define EXTRUDER_DEFAULT_MAX_FEEDRATE 35
        #define EXTRUDER_DEFAULT_MAX_ACCELERATION 1000
        #define EXTRUDER_DEFAULT_EJERK 5.0 // (mm/sec)
        #define INVERT_E0_DIR true
        #define INVERT_E1_DIR true
        #define INVERT_E2_DIR true

        // This defines the number of extruders, set to 2 for DD-Extruder dual
        #define EXTRUDERS 1
#endif
#if EXTRUDER == 3
        // This defines the number of extruders, set to 2 for DD-Extruder dual
        #define EXTRUDERS 2
        #define TEMP_SENSOR_1 1
#endif
#if EXTRUDER >= 4
        #define EXTRUDER_STEPS 110
        #define EXTRUDER_DEFAULT_MAX_FEEDRATE 100
        #define EXTRUDER_DEFAULT_MAX_ACCELERATION 1000
        #define EXTRUDER_DEFAULT_EJERK 5.0 // (mm/sec)
        #define INVERT_E0_DIR true
        #define INVERT_E1_DIR true
        #define INVERT_E2_DIR true

        // This defines the number of extruders, set to 2 for DD-Extruder dual
        #define EXTRUDERS 1
#endif

// Steps 
// 3200/1.25
// 2560 = (((360/1.8) * (1/(1/16)))/1.25h)*(1 / 1)

#if PRINTER_TYPE == 1
        #ifdef PIDTEMPBED
            #define  DEFAULT_bedKp 43.79
            #define  DEFAULT_bedKi .43
            #define  DEFAULT_bedKd 1115.75
        #endif // PIDTEMPBED

	// Disables axis when it's not being used.
	#define DISABLE_X false
	#define DISABLE_Y false
	#define DISABLE_Z true
	#define DISABLE_E false // For all extruders

	// Travel limits after homing
	#define X_MIN_POS 0
	#define Y_MIN_POS 0
	#define Z_MIN_POS 0
	#define X_MAX_POS 200
	#define Y_MAX_POS 200
	#define Z_MAX_POS 115

	#define INVERT_X_DIR false
	#define INVERT_Y_DIR false
	#define INVERT_Z_DIR true

	//// MOVEMENT SETTINGS
	#define HOMING_FEEDRATE {100*60, 100*60, 5*60, 0}  // set the homing speeds (mm/min)

	#define DEFAULT_AXIS_STEPS_PER_UNIT   {80, 80, 1200, EXTRUDER_STEPS}  // default steps per unit
	#define DEFAULT_MAX_FEEDRATE          {600, 600, 15, EXTRUDER_DEFAULT_MAX_FEEDRATE}    // (mm/sec)
	#define DEFAULT_MAX_ACCELERATION      {1500,1500,15, EXTRUDER_DEFAULT_MAX_ACCELERATION}    // X, Y, Z, E maximum start speed for accelerated moves. E default values are good for skeinforge 40+, for older versions raise them a lot.

	#define DEFAULT_ACCELERATION          500   // X, Y, Z and E max acceleration in mm/s^2 for printing moves
	#define DEFAULT_RETRACT_ACCELERATION  500  // X, Y, Z and E max acceleration in mm/s^2 for r retracts

	//
	#define DEFAULT_XYJERK                20.0    // (mm/sec)
	#define DEFAULT_ZJERK                 0.1     // (mm/sec)
	#define DEFAULT_EJERK                 EXTRUDER_DEFAULT_EJERK    // (mm/sec)
#endif

#if PRINTER_TYPE == 2
        #ifdef PIDTEMPBED
            #define  DEFAULT_bedKp 61.53
            #define  DEFAULT_bedKi 0.64
            #define  DEFAULT_bedKd 1477.94
        #endif // PIDTEMPBED

	// Disables axis when it's not being used.
	#define DISABLE_X false
	#define DISABLE_Y false
	#define DISABLE_Z true
	#define DISABLE_E false // For all extruders

	// Travel limits after homing
	#define X_MIN_POS 0
	#define Y_MIN_POS 0
	#define Z_MIN_POS 0

	#define X_MAX_POS 400
	#define Y_MAX_POS 400
	#define Z_MAX_POS 330 // for the optional heated bed, 350 without printbed

	#define INVERT_X_DIR false
	#define INVERT_Y_DIR false
	#define INVERT_Z_DIR true

	//// MOVEMENT SETTINGS
	#define HOMING_FEEDRATE {100*60, 100*60, 5*60, 0}  // set the homing speeds (mm/min)

	#define DEFAULT_AXIS_STEPS_PER_UNIT   {80, 80, 1200, EXTRUDER_STEPS}  // default steps per unit
	#define DEFAULT_MAX_FEEDRATE          {600, 600, 15, EXTRUDER_DEFAULT_MAX_FEEDRATE}    // (mm/sec)
	#define DEFAULT_MAX_ACCELERATION      {1500,1500, 15, EXTRUDER_DEFAULT_MAX_ACCELERATION}    // X, Y, Z, E maximum start speed for accelerated moves. E default values are good for skeinforge 40+, for older versions raise them a lot.

	#define DEFAULT_ACCELERATION          500   // X, Y, Z and E max acceleration in mm/s^2 for printing moves
	#define DEFAULT_RETRACT_ACCELERATION  500   // X, Y, Z and E max acceleration in mm/s^2 for r retracts

	//
	#define DEFAULT_XYJERK                20.0    // (mm/sec)
	#define DEFAULT_ZJERK                 0.1   // (mm/sec)
	#define DEFAULT_EJERK                 EXTRUDER_DEFAULT_EJERK    // (mm/sec)
#endif

#define X_MAX_LENGTH (X_MAX_POS - X_MIN_POS)
#define Y_MAX_LENGTH (Y_MAX_POS - Y_MIN_POS)
#define Z_MAX_LENGTH (Z_MAX_POS - Z_MIN_POS)

// Uncomment the following line to enable CoreXY kinematics
// #define COREXY

// corse Endstop Settings
// Comment this out (using // at the start of the line) to disable the endstop pullup resistors
//#define ENDSTOPPULLUPS

#ifdef ENDSTOPPULLUPS
  // fine Enstop settings: Individual Pullups. will be ignord if ENDSTOPPULLUPS is defined
  #define ENDSTOPPULLUP_XMAX
  #define ENDSTOPPULLUP_XMIN

  #define ENDSTOPPULLUP_YMAX
  #define ENDSTOPPULLUP_YMIN

  #define ENDSTOPPULLUP_ZMAX
  #define ENDSTOPPULLUP_ZMIN
#endif

// The pullups are needed if you directly connect a mechanical endswitch between the signal and ground pins.
const bool X_ENDSTOPS_INVERTING = false; // set to true to invert the logic of the endstops. 
const bool Y_ENDSTOPS_INVERTING = false; // set to true to invert the logic of the endstops. 
const bool Z_ENDSTOPS_INVERTING = false; // set to true to invert the logic of the endstops.

#define DISABLE_MAX_ENDSTOPS

// For Inverting Stepper Enable Pins (Active Low) use 0, Non Inverting (Active High) use 1
#define X_ENABLE_ON 0
#define Y_ENABLE_ON 0
#define Z_ENABLE_ON 0
#define E_ENABLE_ON 0 // For all extruders

// ENDSTOP SETTINGS:
// Sets direction of endstops when homing; 1=MAX, -1=MIN
#define X_HOME_DIR -1
#define Y_HOME_DIR -1
#define Z_HOME_DIR -1

#define min_software_endstops true //If true, axis won't move to coordinates less than HOME_POS.
#define max_software_endstops true  //If true, axis won't move to coordinates greater than the defined lengths below.

#define X_MAX_LENGTH (X_MAX_POS - X_MIN_POS)
#define Y_MAX_LENGTH (Y_MAX_POS - Y_MIN_POS)
#define Z_MAX_LENGTH (Z_MAX_POS - Z_MIN_POS)

// The position of the homing switches
//#define MANUAL_HOME_POSITIONS  // If defined, MANUAL_*_HOME_POS below will be used
//#define BED_CENTER_AT_0_0  // If defined, the center of the bed is at (X=0, Y=0)

//Manual homing switch locations:
#define MANUAL_X_HOME_POS 0
#define MANUAL_Y_HOME_POS 0
#define MANUAL_Z_HOME_POS 0

//// MOVEMENT SETTINGS
#define NUM_AXIS 4

//===========================================================================
//=============================Additional Features===========================
//===========================================================================

// EEPROM
// the microcontroller can store settings in the EEPROM, e.g. max velocity...
// M500 - stores paramters in EEPROM
// M501 - reads parameters from EEPROM (if you need reset them after you changed them temporarily).  
// M502 - reverts to the default "factory settings".  You still need to store them in EEPROM afterwards if you want to.
//define this to enable eeprom support
//#define EEPROM_SETTINGS
//to disable EEPROM Serial responses and decrease program space by ~1700 byte: comment this out:
// please keep turned on if you can.
//#define EEPROM_CHITCHAT

// Preheat Constants
#define PLA_PREHEAT_HOTEND_TEMP 180 
#define PLA_PREHEAT_HPB_TEMP 70
#define PLA_PREHEAT_FAN_SPEED 255		// Insert Value between 0 and 255

#define ABS_PREHEAT_HOTEND_TEMP 240
#define ABS_PREHEAT_HPB_TEMP 100
#define ABS_PREHEAT_FAN_SPEED 255		// Insert Value between 0 and 255

#ifdef ULTRA_LCD
  #define NEWPANEL  //enable this if you have a click-encoder panel
  #define ULTIPANEL
  #define SDSUPPORT
  #define ULTRA_LCD
  #define LCD_WIDTH 20
  #define LCD_HEIGHT 4
#endif

// Increase the FAN pwm frequency. Removes the PWM noise but increases heating in the FET/Arduino
//#define FAST_PWM_FAN

// M240  Triggers a camera by emulating a Canon RC-1 Remote
// Data from: http://www.doc-diy.net/photo/rc-1_hacked/
// #define PHOTOGRAPH_PIN     23

// SF send wrong arc g-codes when using Arc Point as fillet procedure
//#define SF_ARC_FIX

#include "Configuration_adv.h"
#include "thermistortables.h"

#endif //__CONFIGURATION_H
